/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package BallWar;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.MouseAdapter;
import java.awt.geom.AffineTransform;
import java.awt.geom.Dimension2D;
import java.awt.geom.Path2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

/**
 *
 * @author Fabi
 */
public abstract class GameObject extends MouseAdapter
        implements Shape, Serializable {

    
    public static interface ForEachMethod {

        public abstract void doForObject(GameObject o);
    }

    public static interface CollisionListener {

        public abstract void onCollision(GameObject o);
    }
    private static List<GameObject> objects = new LinkedList<GameObject>();
    private static List<GameObject> toEraseObjects = new LinkedList<GameObject>();
    private static List<GameObject> toAddObjects = new LinkedList<GameObject>();
    private static Dimension2D panelBounds;

    /**
     * @return the panelBounds
     */
    public static Dimension2D getPanelBounds() {
        return panelBounds;
    }

    /**
     * @param aPanelBounds the panelBounds to set
     */
    public static void setPanelBounds(Dimension2D aPanelBounds) {
        panelBounds = aPanelBounds;
    }
    
    protected static void eraseLater(GameObject obj) {
        toEraseObjects.add(obj);
    }

    protected static void addLater(GameObject obj) {
        toAddObjects.add(obj);
    }

    public static void doForAll(ForEachMethod method) {
        for (int i = 0; i < objects.size(); i++) {
            method.doForObject(objects.get(i));
        }
        // erase objects
        for (int i = 0; i < toEraseObjects.size(); i++) {
            toEraseObjects.get(i).eraseObject();
        }
        toEraseObjects.clear();
        // add objects
        for (int i = 0; i < toAddObjects.size(); i++) {
            toAddObjects.get(i).add();
        }
        toAddObjects.clear();
    }

    public static void testCollision(final GameObject theObject, final CollisionListener list) {
        doForAll(new ForEachMethod() {

            @Override
            public void doForObject(GameObject o) {
                if (o != theObject) {
                    if (theObject.getBounds2D().intersects(o.getBounds2D())) {
                        list.onCollision(o);
                    }
                }
            }
        });
    }

    public static void saveInFile(File f) {
        ObjectOutputStream out = null;
        try {
            out = new ObjectOutputStream(new FileOutputStream(f));
            final List<GameObject> list = new LinkedList<GameObject>();
            doForAll(new ForEachMethod() {

                @Override
                public void doForObject(GameObject o) {
                    list.add(o);
                }
            });
            out.writeObject(list);
        } catch (IOException ex) {
            throw new RuntimeException("Can't save objects!", ex);
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    public static void loadFromFile(File f) {
        ObjectInputStream in = null;
        try {
            in = new ObjectInputStream(new FileInputStream(f));
            final List<GameObject> list = (List<GameObject>) in.readObject();
            for (GameObject o : list) {
                objects.add(o);
                o.init();
            }
        } catch (IOException ex) {
            throw new RuntimeException("Can't save objects!", ex);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    public static void updateAll() {
        doForAll(new ForEachMethod() {

            @Override
            public void doForObject(GameObject o) {
                o.updateObject();
            }
        });
    }

    public static void paintAll(final Graphics2D g2d, int width, int height) {
        g2d.clearRect(0, 0, width, height);
        doForAll(new ForEachMethod() {

            @Override
            public void doForObject(GameObject o) {
                o.paintObject(g2d);
            }
        });
    }

    public static void eraseAll() {
        doForAll(new ForEachMethod() {

            @Override
            public void doForObject(GameObject o) {
                GameObject.eraseLater(o);
            }
        });
    }

    private Shape shape;
    private Color color;

    public GameObject() {
        setShape (null); // use dummy shape
        init();
    }

    /**
     * @return the shape
     */
    protected Shape getShape() {
        return shape;
    }

    /**
     * @param shape the shape to set
     */
    protected void setShape(Shape shape) {
         if (shape != null) {
            this.shape = shape;
        } else {
            this.shape = new DummyShape();
        }
    }

    /**
     * @return the color
     */
    public Color getColor() {
        return color;
    }

    /**
     * @param color the color to set
     */
    public void setColor(Color color) {
        this.color = color;
    }

    private void add() {
        if (!objects.contains(this)) {
            objects.add(this);
        }
    }

    private void eraseObject() {
        objects.remove(this);
    }
    

    public void paintObject(Graphics2D g2d){
        g2d.setColor(color);
        g2d.fill(shape);
    }

    public abstract void updateObject();

    public abstract void init();

    public Point2D getPosition (){
        Rectangle2D bounds = getShape().getBounds2D();
        return new Point2D.Double (bounds.getCenterX(), bounds.getCenterY());
    }
    // dummy implementation (for some invisible objects) of the abstract methods of shape    

    private static class DummyShape implements Shape, Serializable {

        @Override
        public Rectangle getBounds() {
            return new Rectangle();
        }

        @Override
        public Rectangle2D getBounds2D() {
            return getBounds();
        }

        @Override
        public boolean contains(double x, double y) {
            return false;
        }

        @Override
        public boolean contains(Point2D p) {
            return false;
        }

        @Override
        public boolean intersects(double x, double y, double w, double h) {
            return false;
        }

        @Override
        public boolean intersects(Rectangle2D r) {
            return false;
        }

        @Override
        public boolean contains(double x, double y, double w, double h) {
            return false;
        }

        @Override
        public boolean contains(Rectangle2D r) {
            return false;
        }

        @Override
        public PathIterator getPathIterator(AffineTransform at) {
            return getPathIterator(at, 0.0f);
        }

        @Override
        public PathIterator getPathIterator(AffineTransform at, double flatness) {
            Path2D p = new Path2D.Float();
            p.moveTo(0.0, 0.0);
            p.closePath();
            return p.getPathIterator(null);
        }
    }

    // setShape(getShape() ) uses dummy shape if getShape() == null
    @Override
    public Rectangle getBounds() {
        setShape(getShape());
        return getShape().getBounds();
    }

    @Override
    public Rectangle2D getBounds2D() {
        setShape(getShape());
        return getShape().getBounds2D();
    }

    @Override
    public boolean contains(double x, double y) {
        setShape(getShape());
        return getShape().contains(x, y);
    }

    @Override
    public boolean contains(Point2D p) {
        setShape(getShape());
        return getShape().contains(p);
    }

    @Override
    public boolean intersects(double x, double y, double w, double h) {
        setShape(getShape());
        return getShape().intersects(x, y, w, h);
    }

    @Override
    public boolean intersects(Rectangle2D r) {
        setShape(getShape());
        return getShape().intersects(r);
    }

    @Override
    public boolean contains(double x, double y, double w, double h) {
        setShape(getShape());
        return getShape().contains(x, y, w, h);
    }

    @Override
    public boolean contains(Rectangle2D r) {
        setShape(getShape());
        return getShape().contains(r);
    }

    @Override
    public PathIterator getPathIterator(AffineTransform at) {
        setShape(getShape());
        return getShape().getPathIterator(at);
    }

    @Override
    public PathIterator getPathIterator(AffineTransform at, double flatness) {
        setShape(getShape());
        return getShape().getPathIterator(at, flatness);
    }
}
