/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package BallWar;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Dimension2D;
import java.awt.geom.Point2D;
import java.io.File;
import javax.swing.Timer;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

/**
 *
 * @author Fabi
 */
public class GameStarter extends Application {

    private static final int FOCUS_TIMER_DELAY = 15;
    private Player player;
    private GamePanel panel;
    private GameScrollPane scroller;
    private BlockChooserPanel bcp;
    private Dimension d;
    private Ball ball;
    private JFrame frame;
    private boolean showFrame = true;
    private Level theChosenLevel;

    @Override
    protected void exit() {
        if (frame != null) {
            frame.dispose();
        }
        GameObject.eraseAll();
        if (ball != null) {
            if (ball.getPoints() < theChosenLevel.getMinPoints()) {
                JOptionPane.showMessageDialog(null,
                        "Ihre Punktzahl ist leider unter der erforderlichen"
                        + " Mindestpunktzahl von "
                        + theChosenLevel.getMinPoints() + " Punkten.");
            } else {
                JOptionPane.showMessageDialog(null,
                        "Gratualation! Sie haben " + ball.getPoints() + " Punkte!");
            }
        }
    }

    public GameStarter() {
        frame = new JFrame("GameStarter");
        if (!readLevels()) {
            return;
        }
        if (!showFrame) {
            return;
        }
        frame.setResizable(false);
        frame.setLayout(new BorderLayout());
        panel = new GamePanel(d);
        scroller = new GameScrollPane(panel);
        frame.add(scroller, BorderLayout.CENTER);
        bcp = new BlockChooserPanel();
        frame.add(bcp, BorderLayout.NORTH);
        frame.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
        frame.pack();
    }

    @Override
    protected void init() {
    }

    @Override
    protected void start() {
        if (!showFrame) {
            return;
        }

        player = new Player(bcp);
        GameObject.addLater(player);
        // ball in focus
        new Timer(FOCUS_TIMER_DELAY, new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                scroller.getVerticalScrollBar().setValue((int) (ball.getPosition().getY()
                        - Toolkit.getDefaultToolkit().getScreenSize().height / 2));
            }
        }).start();
        GameObject.addLater(ball);

        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                frame.setVisible(true);
            }
        });
    }

    private boolean readLevels() {
        if (levelFile.exists() == false) {
            JOptionPane.showMessageDialog(null,
                    "Keine Level! Kann Spiel leider nicht starten.", "Fehler",
                    JOptionPane.ERROR_MESSAGE);
            exitApplication();
            return false;
        }
        LevelList levels = LevelList.loadLevelList(levelFile);
        LevelChooser chooser = new LevelChooser(levels.getLevels());
        theChosenLevel = null;
        while ((theChosenLevel = chooser.getLevel()) == null) {
            if (!chooser.isVisible()) {
                break;
            }
        }
        if (theChosenLevel == null) {
            showFrame = false;
            return false;
        }
        Dimension2D size = theChosenLevel.getSize();
        d = new Dimension((int) size.getWidth(), (int) size.getHeight());
        frame.setTitle(theChosenLevel.getName());
        for (GameObject o : theChosenLevel) {
            GameObject.addLater(o);
        }
        StartPoint sp = theChosenLevel.getStartPosition();
        Point2D p = sp.getPosition();
        ball = new Ball((int) p.getX(), (int) p.getY());
        final EndRect er = theChosenLevel.getEndRect();
        new Timer(100, new ActionListener() {

            private boolean stopped = false;

            public void actionPerformed(ActionEvent e) {
                if (stopped) {
                    return;
                }
                if (ball.intersects(er.getBounds2D())) {
                    exitApplication();
                    stopped = true;
                } else {
                    final MutablePrimitive<Boolean> exists = new MutablePrimitive<Boolean>(false);
                    GameObject.doForAll(new GameObject.ForEachMethod() {

                        public void doForObject(GameObject o) {
                            if (o == ball) {
                                exists.setObject(true);
                            }
                        }
                    });
                    if (!exists.getObject()) {
                        exitApplication();
                        stopped = true;
                    }
                }
            }
        }).start();
        return true;
    }
    public static final File levelFile = new File("levels.lvl");
}
